// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

import java.awt.Frame;
import java.awt.Panel;
import java.awt.Button;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.*;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Hashtable;

import org.isam.exehda.Exehda;
import org.isam.exehda.services.Collector;
import org.isam.exehda.services.Collector.Sensor;
import org.isam.exehda.services.Collector.MonitoringData;
import org.isam.exehda.services.Collector.ConsumerId;



public class CollectorDemo
    extends Frame
    implements WindowListener, ActionListener
{
    private Panel sensorsPane;
    private Hashtable labelsBySensorName;
    private Collector collector;
    private Sensor[] sensors;
    private ConsumerId id;

    
    public void windowOpened(WindowEvent windowEvent) {}

    public void windowClosing(WindowEvent windowEvent)
        {
            System.exit(0);
        }

    public void windowClosed(WindowEvent windowEvent) {}

    public void windowIconified(WindowEvent windowEvent) {}

    public void windowDeiconified(WindowEvent windowEvent) {}

    public void windowActivated(WindowEvent windowEvent) {}

    public void windowDeactivated(WindowEvent windowEvent) {}
    
    
    public CollectorDemo()
        {
            super("Collector Demo");
            addWindowListener(this);
            
            Dimension d = new Dimension(200, 100);
            
            sensorsPane = new Panel();
            sensorsPane.setLayout(new GridBagLayout());

            this.setBounds(50,50,100,100);
            this.add(sensorsPane);

            Button b = new Button("update");
            this.add(b, BorderLayout.NORTH);
            b.addActionListener(this);

            collector = (Collector) Exehda.getService(Collector.SERVICE_NAME);
                // FIX ME: registering a call back would be more efficient
            id = collector.addConsumer(null);

            sensors = collector.getSensors();
            for ( int i=0; i< sensors.length; i++ ) {
                collector.setSensorEnabled(this.id, sensors[i], true);
            }
            
            
            labelsBySensorName = new Hashtable();
        }

    public void addSensor(String name, String value)
        {
            Label sv = (Label) labelsBySensorName.get(name);

            if (sv == null) {
                Label sn = new Label(name);
                sn.setFont(new Font("helvetica", Font.BOLD, 10));
                       
                sv = new Label(value);
                sv.setFont(new Font("helvetica", Font.PLAIN, 10));

                GridBagConstraints gbc = new GridBagConstraints();
                gbc.fill = java.awt.GridBagConstraints.HORIZONTAL;
                gbc.insets = new java.awt.Insets(1, 5, 1, 6);
            
                sensorsPane.add(sn, gbc);

                gbc.gridwidth = java.awt.GridBagConstraints.REMAINDER;
                sensorsPane.add(sv, gbc);

                labelsBySensorName.put(name, sv);
            }
            else {
                sv.setText(value);
            }
        }

    public void actionPerformed(ActionEvent ev)
        {
            updateSensors();
        }

    public void updateSensors()
        {
//             System.out.println("Entering update sensors...");
//            mon.quantumExpired();
            
            for ( int i=0; i< sensors.length; i++) {
                MonitoringData data = collector.probeSensor(id, sensors[i], null);
                addSensor(sensors[i].getName().getSimpleName(),
                          data.getString());
            }

//             System.out.println("Leaving update sensors.");
        }

    public static void
    main( String[] args ) throws Exception
        {
            CollectorDemo frame = new CollectorDemo();

            frame.updateSensors();
            frame.pack();
            frame.setVisible(true);

            for ( int i=0; i<100; i++) {
                Thread.currentThread().sleep(1000);
                frame.updateSensors();
            }
        }
}
