
import org.isam.exehda.Exehda;
import org.isam.exehda.services.ContextManager;
import org.isam.exehda.services.ContextManager.Context;
import org.isam.exehda.services.ContextManager.ContextState;
import org.isam.exehda.services.ContextManager.ContextListener;

public class CtxManagerDemo
{
    static final String CTXDEF = ( ""
        + "<CONTEXT n='load'>\n"
        + "  <STATES>\n"
        + "    <STATE n='free for chat'/>\n"
        + "    <STATE n='occupied'/>\n"
        + "    <STATE n='do not disturb!'/>\n"
        + "  </STATES>\n"
        + "  <FILTER syntax='xml:NumericRangeMapping'>\n"
        + "    <INDEX>\n"
        + "      <SENSOR n='CPU_OCCUP_IDLE'/>\n"
        + "    </INDEX>\n"
        + "    <MAPPINGS>\n"
        + "      <RANGE ub='30' state='do not disturb!'/>\n"
        + "      <RANGE lbo='30' ub='60' state='occupied'/>\n"
        + "      <RANGE lbo='60' state='free for chat'/>\n"
        + "      <DEFAULT state='occupied'/>\n"
        + "    </MAPPINGS>\n"
        + "  </FILTER>\n"
        + "</CONTEXT>\n").replace('\'','"');

    static final String CTXDEF2 = ( ""
        + "<CONTEXT n='gui'>\n"
        + "  <STATES>\n"
        + "    <STATE n='console'/>\n"
        + "    <STATE n='pda'/>\n"
        + "    <STATE n='desktop'/>\n"
        + "  </STATES>\n"
        + "  <FILTER syntax='xml:StringMapping'>\n"
        + "    <INDEX>\n"
        + "      <SENSOR n='runtime.gui.awt.mode'/>\n"
        + "    </INDEX>\n"
        + "    <MAPPINGS>\n"
        + "      <MATCH op='equals' v='none' ignoreCase='true' state='console'/>\n"
        + "      <MATCH op='equals' v='awt-low res' ignoreCase='true' state='pda'/>\n"
        + "      <MATCH op='prefix' v='swing' ignoreCase='true' state='desktop'/>\n"
        + "      <DEFAULT state='console'/>\n"
        + "    </MAPPINGS>\n"
        + "  </FILTER>\n"
        + "</CONTEXT>\n").replace('\'','"');
    
    public static void
    main(String[] args)
        throws Exception
        {
            ContextManager ctxman = (ContextManager)
                Exehda.getService(ContextManager.SERVICE_NAME);

            Context ctx = ctxman.createContext(CTXDEF);
            Context ctx2 = ctxman.createContext(CTXDEF2);

            ContextListener l = new ContextListener() {
                    public void contextChanged(ContextState prev, ContextState curr) {
                        System.out.println("Context changed from "+prev+" to "+curr);
                    }
                };

            System.out.println("Registering context listener...");
            
            ctxman.addContextListener(l, ctx);
            ctxman.addContextListener(l, ctx2);

            System.out.println("Waiting for context events...");
            
            Thread.currentThread().sleep(60000);

            System.out.println("Cleaning up...");
            
            ctxman.removeContextListener(l, ctx);
            ctxman.removeContextListener(l, ctx2);
            ctxman.releaseContext(ctx);
        }
}
