
import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ObjectId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.Executor;
import org.isam.exehda.services.ObjectSeed.Activator;
import org.isam.exehda.services.ObjectSeed.MarshaledOX;
import org.isam.exehda.services.OXManager;
import org.isam.exehda.services.OXManager.OXHandle;

public class ExecutorDemo
{
    public static class Walker
        implements Runnable, java.io.Serializable
    {
        public Walker( String name )
            {
                this.name = name;
                this.count = 0;
                this.where = null;
            }

        public void run()
            {
                System.out.println(name+" running!");
                while ( ! Thread.interrupted() ) {
                    System.out.println(name+" is working at "+where+"... step #"+(++count));
                    sleep(2000);
                }

                System.out.println(name+" interrupted!");
            }

        public void setWhere(String where)
            {
                this.where = where;
            }

        private String where;
        private String name;
        private int count;
    }

    public static class WalkerActivator
        implements Activator
    {
        public void activate( ObjectId oxID, Object obj, MarshaledOX extState )
            {
                Walker walker = (Walker) obj;
                walker.setWhere(HostId.getLocalHost().toExternalForm());
                
                thr = new Thread(walker);
                thr.start();
            }

        public void deactivate( ObjectId oxID, Object obj, MarshaledOX oxState )
            throws InterruptedException
            {
//                 System.out.println("Interrupting thread...");
                thr.interrupt();
//                 System.out.println("Waiting for thread exit...");
                thr.join();
//                 System.out.println("Deactivation completed.");
                thr = null;
            }

        private transient Thread thr;
    }

    public static void
    sleep(long millis)
        {
            try {
                Thread.sleep(millis);
            }
            catch (InterruptedException ie) {
                System.out.println(Thread.currentThread()+ " interrupted!");
                    // stay in interrupted mode, otherwise the thread will never leave
                    // the main loop
                Thread.currentThread().interrupt();
            }
        }

        //////////////////////////////////////////////////////////
        // MAIN
        //////////////////////////////////////////////////////////
    public static void
    main( String[] args )
        {
            if ( args.length < 3 ) {
                System.out.println("Missing arguments");
                System.out.println("Usage: ExecutorDemo <hostid> <hostid> <hostid>...");
                    //
                    // hostid := hostid:<hexvalue>.<celname>
                    //
                    // e.g.: hostid:74cc.somecell
                    //
                return;
            }

            Executor executor = (Executor) Exehda.getService(Executor.SERVICE_NAME);

            System.out.println("Current application is:"+ executor.currentApplication());

            HostId h1 = HostId.parseId(args[0]);
            HostId h2 = HostId.parseId(args[1]);
            HostId h3 = HostId.parseId(args[2]);

            ObjectId walkerID = executor.createObject(Walker.class,
                                                      new Object[] { "Walker#"+h1 },
                                                      new WalkerActivator(),
                                                      h1);
            
            System.out.println("Create object completed....");

            for( int i=0; i<10; i++ ) {
                sleep(10000);
                executor.moveObject( walkerID, h2 );

                System.out.println("Moved to "+h2);

                sleep(10000);
            
                executor.moveObject( walkerID, h3 );

                System.out.println("Moved to "+h3);
            }
        }
}

